@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import logging
-from typing import TYPE_CHECKING, overload
+from typing import TYPE_CHECKING, Optional, Union, cast, overload
 
 from pydoll.commands import (
     DomCommands,
@@ -15,7 +15,7 @@ from pydoll.exceptions import ElementNotFound, WaitElementTimeout
 if TYPE_CHECKING:
     from typing import Literal, Optional, Union
 
-    from pydoll.elements.web_element import WebElement
+    from pydoll.elements.web_element import WebElement, _IFrameContext
     from pydoll.protocol.base import Command, T_CommandParams, T_CommandResponse
     from pydoll.protocol.dom.methods import DescribeNodeResponse
     from pydoll.protocol.dom.types import Node
@@ -70,6 +70,20 @@ class FindElementsMixin:
         **attributes,
     ) -> WebElement: ...
 
+    @overload
+    async def find(
+        self,
+        id: Optional[str] = ...,
+        class_name: Optional[str] = ...,
+        name: Optional[str] = ...,
+        tag_name: Optional[str] = ...,
+        text: Optional[str] = ...,
+        timeout: int = ...,
+        find_all: Literal[False] = False,
+        raise_exc: Literal[False] = False,
+        **attributes,
+    ) -> Optional[WebElement]: ...
+
     @overload
     async def find(
         self,
@@ -98,20 +112,6 @@ class FindElementsMixin:
         **attributes,
     ) -> Optional[list[WebElement]]: ...
 
-    @overload
-    async def find(
-        self,
-        id: Optional[str] = ...,
-        class_name: Optional[str] = ...,
-        name: Optional[str] = ...,
-        tag_name: Optional[str] = ...,
-        text: Optional[str] = ...,
-        timeout: int = ...,
-        find_all: Literal[False] = False,
-        raise_exc: Literal[False] = False,
-        **attributes,
-    ) -> Optional[WebElement]: ...
-
     @overload
     async def find(
         self,
@@ -345,7 +345,19 @@ class FindElementsMixin:
             ElementNotFound: If element not found and raise_exc=True.
         """
         logger.debug(f'_find_element(): by={by}, value={value}, raise_exc={raise_exc}')
-        if hasattr(self, '_object_id'):
+        iframe_context = None
+        if getattr(self, 'is_iframe', False):
+            element_self = cast('WebElement', self)
+            iframe_context = await element_self.iframe_context
+
+        if iframe_context:
+            command = self._get_find_element_command(
+                by,
+                value,
+                object_id=iframe_context.document_object_id or '',
+                execution_context_id=iframe_context.execution_context_id,
+            )
+        elif hasattr(self, '_object_id'):
             command = self._get_find_element_command(by, value, self._object_id)
         else:
             command = self._get_find_element_command(by, value)
@@ -363,7 +375,9 @@ class FindElementsMixin:
         object_id = response_for_command['result']['result']['objectId']
         attributes = await self._get_object_attributes(object_id=object_id)
         logger.debug(f'_find_element() found object_id={object_id}')
-        return create_web_element(object_id, self._connection_handler, by, value, attributes)
+        element = create_web_element(object_id, self._connection_handler, by, value, attributes)
+        self._apply_iframe_context_to_element(element, iframe_context)
+        return element
 
     async def _find_elements(self, by: By, value: str, raise_exc: bool = True) -> list[WebElement]:
         """
@@ -385,7 +399,19 @@ class FindElementsMixin:
             ElementNotFound: If no elements found and raise_exc=True.
         """
         logger.debug(f'_find_elements(): by={by}, value={value}, raise_exc={raise_exc}')
-        if hasattr(self, '_object_id'):
+        iframe_context = None
+        if getattr(self, 'is_iframe', False):
+            element_self = cast('WebElement', self)
+            iframe_context = await element_self.iframe_context
+
+        if iframe_context:
+            command = self._get_find_elements_command(
+                by,
+                value,
+                object_id=iframe_context.document_object_id or '',
+                execution_context_id=iframe_context.execution_context_id,
+            )
+        elif hasattr(self, '_object_id'):
             command = self._get_find_elements_command(by, value, self._object_id)
         else:
             command = self._get_find_elements_command(by, value)
@@ -421,9 +447,9 @@ class FindElementsMixin:
             tag_name = node_description.get('nodeName', '').lower()
             attributes.extend(['tag_name', tag_name])
 
-            elements.append(
-                create_web_element(object_id, self._connection_handler, by, value, attributes)
-            )
+            child = create_web_element(object_id, self._connection_handler, by, value, attributes)
+            self._apply_iframe_context_to_element(child, iframe_context)
+            elements.append(child)
         logger.debug(f'_find_elements() returning {len(elements)} elements')
         return elements
 
@@ -432,6 +458,10 @@ class FindElementsMixin:
         Get attributes of a DOM node.
         """
         node_description = await self._describe_node(object_id=object_id)
+        if not node_description:
+            # If the node couldn't be described (e.g., object id doesn't reference a Node),
+            # return minimal attributes to keep the flow stable.
+            return ['tag_name', '']
         attributes = node_description.get('attributes', [])
         tag_name = node_description.get('nodeName', '').lower()
         attributes.extend(['tag_name', tag_name])
@@ -457,6 +487,11 @@ class FindElementsMixin:
             f'_get_by_and_value(): id={id}, class_name={class_name}, name={name}, '
             f'tag_name={tag_name}, text={text}, attrs={attributes}'
         )
+        xpath_raw = attributes.get('xpath')
+        if isinstance(xpath_raw, str) and xpath_raw:
+            logger.debug(f'Explicit XPath provided; using raw expression: {xpath_raw}')
+            return By.XPATH, xpath_raw
+
         simple_selectors = {
             'id': id,
             'class_name': class_name,
@@ -541,15 +576,58 @@ class FindElementsMixin:
         response: DescribeNodeResponse = await self._execute_command(
             DomCommands.describe_node(object_id=object_id)
         )
-        return response['result']['node']
+        if 'error' in response:
+            # Return empty node structure when CDP reports that the objectId
+            # doesn't reference a Node or any other describe error occurs.
+            return {}
+        return response.get('result', {}).get('node', {})
+
+    def _apply_iframe_context_to_element(
+        self, element: WebElement, iframe_context: _IFrameContext | None
+    ) -> None:
+        """
+        Propagate iframe context to the newly created element.
+        - If the element is also an iframe, configure session routing.
+        - Otherwise, inject the iframe's own context.
+        """
+        if not iframe_context:
+            return
+        if getattr(element, 'is_iframe', False):
+            routing_handler = iframe_context.session_handler or self._connection_handler
+            element._routing_session_handler = routing_handler
+            element._routing_session_id = iframe_context.session_id
+            element._routing_parent_frame_id = iframe_context.frame_id
+            return
+        element._iframe_context = iframe_context
+
+    def _resolve_routing(self) -> tuple[ConnectionHandler, Optional[str]]:
+        """
+        Resolve handler and sessionId for the current context (iframe routed or default).
+        """
+        iframe_context = getattr(self, '_iframe_context', None)
+        if iframe_context and getattr(iframe_context, 'session_handler', None):
+            return iframe_context.session_handler, getattr(iframe_context, 'session_id', None)
+        routing_handler = getattr(self, '_routing_session_handler', None)
+        if routing_handler is not None:
+            return routing_handler, getattr(self, '_routing_session_id', None)
+        return self._connection_handler, None
 
     async def _execute_command(
         self, command: Command[T_CommandParams, T_CommandResponse]
     ) -> T_CommandResponse:
-        """Execute CDP command via connection handler (60s timeout)."""
-        return await self._connection_handler.execute_command(command, timeout=60)
+        """Execute CDP command via resolved handler (60s timeout)."""
+        handler, session_id = self._resolve_routing()
+        if session_id:
+            command['sessionId'] = session_id
+        return await handler.execute_command(command, timeout=60)
 
-    def _get_find_element_command(self, by: By, value: str, object_id: str = ''):
+    def _get_find_element_command(
+        self,
+        by: By,
+        value: str,
+        object_id: str = '',
+        execution_context_id: Optional[int] = None,
+    ):
         """
         Create CDP command for finding single element.
 
@@ -579,18 +657,29 @@ class FindElementsMixin:
                 return_by_value=False,
             )
         elif by == By.XPATH:
-            command = self._get_find_element_by_xpath_command(value, object_id)
+            command = self._get_find_element_by_xpath_command(
+                value, object_id=object_id, execution_context_id=execution_context_id
+            )
         elif by == By.NAME:
             command = self._get_find_element_by_xpath_command(
-                f'//*[@name="{escaped_value}"]', object_id
+                f'//*[@name="{escaped_value}"]',
+                object_id=object_id,
+                execution_context_id=execution_context_id,
             )
         else:
             command = RuntimeCommands.evaluate(
-                expression=Scripts.QUERY_SELECTOR.replace('{selector}', selector)
+                expression=Scripts.QUERY_SELECTOR.replace('{selector}', selector),
+                context_id=execution_context_id,
             )
         return command
 
-    def _get_find_elements_command(self, by: By, value: str, object_id: str = ''):
+    def _get_find_elements_command(
+        self,
+        by: By,
+        value: str,
+        object_id: str = '',
+        execution_context_id: Optional[int] = None,
+    ):
         """
         Create CDP command for finding multiple elements.
 
@@ -617,14 +706,22 @@ class FindElementsMixin:
                 return_by_value=False,
             )
         elif by == By.XPATH:
-            command = self._get_find_elements_by_xpath_command(value, object_id)
+            command = self._get_find_elements_by_xpath_command(
+                value, object_id=object_id, execution_context_id=execution_context_id
+            )
         else:
             command = RuntimeCommands.evaluate(
-                expression=Scripts.QUERY_SELECTOR_ALL.replace('{selector}', selector)
+                expression=Scripts.QUERY_SELECTOR_ALL.replace('{selector}', selector),
+                context_id=execution_context_id,
             )
         return command
 
-    def _get_find_element_by_xpath_command(self, xpath: str, object_id: str):
+    def _get_find_element_by_xpath_command(
+        self,
+        xpath: str,
+        object_id: str,
+        execution_context_id: Optional[int] = None,
+    ):
         """
         Create CDP command specifically for XPath single element finding.
 
@@ -646,10 +743,15 @@ class FindElementsMixin:
             )
         else:
             script = Scripts.FIND_XPATH_ELEMENT.replace('{escaped_value}', escaped_value)
-            command = RuntimeCommands.evaluate(expression=script)
+            command = RuntimeCommands.evaluate(expression=script, context_id=execution_context_id)
         return command
 
-    def _get_find_elements_by_xpath_command(self, xpath: str, object_id: str):
+    def _get_find_elements_by_xpath_command(
+        self,
+        xpath: str,
+        object_id: str,
+        execution_context_id: Optional[int] = None,
+    ):
         """
         Create CDP command specifically for XPath multiple element finding.
 
@@ -671,7 +773,7 @@ class FindElementsMixin:
             )
         else:
             script = Scripts.FIND_XPATH_ELEMENTS.replace('{escaped_value}', escaped_value)
-            command = RuntimeCommands.evaluate(expression=script)
+            command = RuntimeCommands.evaluate(expression=script, context_id=execution_context_id)
         return command
 
     @staticmethod
