@@ -103,6 +103,12 @@ class ProtocolException(PydollException):
     message = 'A protocol error occurred'
 
 
+class TopLevelTargetRequired(ProtocolException):
+    """Raised when a command can only be executed on top-level targets."""
+
+    message = 'Command can only be executed on top-level targets.'
+
+
 class InvalidCommand(ProtocolException):
     """Raised when an invalid command is sent to the browser."""
 
@@ -283,6 +289,18 @@ class NetworkEventsNotEnabled(PydollException):
     message = 'Network events not enabled'
 
 
+class RequestException(PydollException):
+    """Base class for exceptions related to HTTP requests."""
+
+    message = 'An HTTP request error occurred'
+
+
+class HTTPError(RequestException):
+    """Exception raised for HTTP error responses (4xx and 5xx status codes)."""
+
+    message = 'An HTTP error occurred'
+
+
 class ScriptException(PydollException):
     """Base class for exceptions related to JavaScript execution."""
 
